@@ -1,33 +1,26 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright     1999-2006  David Faure <faure@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 1999-2006 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KSERVICE_H
 #define KSERVICE_H
 
 #include "kserviceaction.h"
+#include <QCoreApplication>
 #include <QStringList>
 #include <QVariant>
+#include <ksycocaentry.h>
+
+// Exclude includes that are not needed when deprecations are disabled
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 94)
 #include <KPluginFactory>
 #include <KPluginLoader>
-#include <ksycocaentry.h>
-#include <QCoreApplication>
+#endif
 
 class KServiceType;
 class QDataStream;
@@ -40,7 +33,7 @@ class KServicePrivate;
  * @class KService kservice.h <KService>
  *
  * Represent a service, like an application or plugin
- * bound to one or several mimetypes (or servicetypes) as written
+ * bound to one or several MIME types (or servicetypes) as written
  * in its desktop entry file.
  *
  * The starting point you need is often the static methods, like createInstance().
@@ -90,7 +83,7 @@ public:
 
     KService(const KService &other);
 
-    virtual ~KService();
+    ~KService() override;
 
     /**
      * Services are either applications (executables) or dlopened libraries (plugins).
@@ -133,12 +126,20 @@ public:
      *         or QString() if not set
      */
     QString terminalOptions() const;
+
     /**
-     * Checks whether the service runs on a discrete graphics card
-     * @return true if the service has to run under a discrete graphics card
+     * Returns @c true if the service inidicates that it's preferred to run
+     * the application on a discrete graphics card, otherwise return @c false.
+     *
+     * In releases older than 5.86 this methoed checked for the @c X-KDE-RunOnDiscreteGpu
+     * key in the .desktop file represented by this service; starting from 5.86 this method
+     * now also checks for @c PrefersNonDefaultGPU key (added to the Freedesktop.org desktop
+     * entry spec in version 1.4 of the spec).
+     *
      * @since 5.30
      */
     bool runOnDiscreteGpu() const;
+
     /**
      * Checks whether the service runs with a different user id.
      * @return true if the service has to be run under a different uid.
@@ -177,6 +178,7 @@ public:
      */
     QString storageId() const;
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 102)
     /**
      * Describes the D-Bus Startup type of the service.
      * @li None - This service has no D-Bus support
@@ -188,12 +190,18 @@ public:
      *             the PID of the process.
      */
     enum DBusStartupType { DBusNone = 0, DBusUnique, DBusMulti };
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 102)
     /**
      * Returns the DBUSStartupType supported by this service.
      * @return the DBUSStartupType supported by this service
+     *
+     * @deprecated since 5.102, no known users.
      */
+    KSERVICE_DEPRECATED_VERSION(5, 102, "No known users")
     DBusStartupType dbusStartupType() const;
+#endif
 
 #if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 63)
     /**
@@ -249,9 +257,9 @@ public:
     QStringList categories() const;
 
     /**
-     * Returns the list of mime types that this service supports.
-     * Note that this doesn't include inherited mimetypes,
-     * only the mimetypes types listed in the .desktop file.
+     * Returns the list of MIME types that this service supports.
+     * Note that this doesn't include inherited MIME types,
+     * only the MIME types listed in the .desktop file.
      * @since 4.8.3
      */
     QStringList mimeTypes() const;
@@ -259,7 +267,7 @@ public:
     /**
      * Returns the service types that this service supports.
      * @return the list of service types that are supported
-     * Note that this doesn't include inherited servicetypes or mimetypes,
+     * Note that this doesn't include inherited servicetypes or MIME types,
      * only the service types listed in the .desktop file.
      */
     QStringList serviceTypes() const;
@@ -274,8 +282,8 @@ public:
     bool hasServiceType(const QString &serviceTypePtr) const;
 
     /**
-     * Checks whether the service supports this mime type
-     * @param mimeType The name of the mime type you are
+     * Checks whether the service supports this MIME type
+     * @param mimeType The name of the MIME type you are
      *        interested in determining whether this service supports.
      * @since 4.6
      */
@@ -340,7 +348,10 @@ public:
      * @deprecated since 5.0, use showInCurrentDesktop()
      */
     KSERVICE_DEPRECATED_VERSION(5, 0, "Use KService::showInCurrentDesktop()")
-    bool showInKDE() const { return showInCurrentDesktop(); }
+    bool showInKDE() const
+    {
+        return showInCurrentDesktop();
+    }
 #endif
 
     /**
@@ -348,7 +359,7 @@ public:
      * (including in context menus).
      * @return true if the service should be shown.
      *
-     * KMimeTypeTrader honors this and removes such services
+     * KApplicationTrader honors this and removes such services
      * from its results.
      *
      * @since 5.0
@@ -364,19 +375,32 @@ public:
      */
     bool showOnCurrentPlatform() const;
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 87)
     /**
      * Name of the application this service belongs to.
      * (Useful for e.g. plugins)
      * @return the parent application, or QString() if not set
+     * @deprecated Since 5.87, the concept of parent apps is deprecated. Plugins should use KPluginMetaData instead of
+     * KService and a dedicated namespace if the plugins are only for one app relevant.
      */
+    KSERVICE_DEPRECATED_VERSION(5, 87, "See API docs")
     QString parentApp() const;
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 87)
     /**
      * The keyword to be used when constructing the plugin using KPluginFactory. The keyword is
      * defined with X-KDE-PluginKeyword in the .desktop file and with registerPlugin<T>(keyword)
      * in the K_PLUGIN_FACTORY macro when implementing the plugin.
+     * @deprecated Since 5.87, the metadata should be embedded in the actual plugin. Consequently
+     * this property is obsolete. In case there is only one plugin in the library the usage is not needed.
+     * In case there are different base classes registered the keyword is not needed too.
+     * If there are multiple classes of a common base class registered they should be split up
+     * in separate libs.
      */
+    KSERVICE_DEPRECATED_VERSION(5, 87, "See API docs")
     QString pluginKeyword() const;
+#endif
 
     /**
      * The path to the documentation for this service.
@@ -385,6 +409,7 @@ public:
      */
     QString docPath() const;
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 102)
     /**
      * Returns the requested property.
      *
@@ -392,8 +417,24 @@ public:
      * @param t the assumed type of the property
      * @return the property, or invalid if not found
      * @see KServiceType
+     *
+     * @deprecated since 5.102, use property(QString, QMetaType::Type) instead.
      */
+    KSERVICE_DEPRECATED_VERSION(5, 102, "Use property(QString, QMetaType::Type) instead")
     QVariant property(const QString &_name, QVariant::Type t) const;
+#endif
+
+    /**
+     * Returns the requested property.
+     *
+     * @param _name the name of the property
+     * @param t the assumed type of the property
+     * @return the property, or invalid if not found
+     * @see KServiceType
+     *
+     * @since 5.102
+     */
+    QVariant property(const QString &_name, QMetaType::Type t) const;
 
     using KSycocaEntry::property;
 
@@ -434,6 +475,21 @@ public:
      */
     void setExec(const QString &exec);
 
+    /**
+     * Overrides the "Path=" line of the service.
+     *
+     * If @ref workingDir is not empty, its value will override the one
+     * the one set when this service was created.
+     *
+     * Please note that @ref entryPath is also cleared so the service
+     * will no longer be associated with a specific config file.
+     *
+     * @internal
+     * @param workingDir
+     * @since 5.79
+     */
+    void setWorkingDirectory(const QString &workingDir);
+
     /**
      * Find a service based on its path as returned by entryPath().
      * It's usually better to use serviceByStorageId() instead.
@@ -509,10 +565,10 @@ public:
      *        of the reservedMenuIds
      * @return The path to use for the new KService.
      */
-    static QString newServicePath(bool showInMenu, const QString &suggestedName,
-                                  QString *menuId = nullptr,
-                                  const QStringList *reservedMenuIds = nullptr);
+    static QString newServicePath(bool showInMenu, const QString &suggestedName, QString *menuId = nullptr, const QStringList *reservedMenuIds = nullptr);
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 86)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * This template allows to load the library for the specified service and ask the
      * factory to create an instance of the given template type.
@@ -524,14 +580,21 @@ public:
      *
      * @return A pointer to the newly created object or a null pointer if the
      *         factory was unable to create an object of the given type.
+     *
+     * @see KPluginFactory::instantiatePlugin
+     * @deprecated Since 5.86, Use KPluginMetaData/KPluginFactory or QPluginloader instead
      */
-    template <class T>
-    T *createInstance(QObject *parent = nullptr,
-                      const QVariantList &args = QVariantList(), QString *error = nullptr) const
+    template<class T>
+    KSERVICE_DEPRECATED_VERSION(5, 86, "Use KPluginMetaData/KPluginFactory or QPluginloader instead")
+    T *createInstance(QObject *parent = nullptr, const QVariantList &args = QVariantList(), QString *error = nullptr) const
     {
         return createInstance<T>(nullptr, parent, args, error);
     }
+#endif
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 86)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * This template allows to load the library for the specified service and ask the
      * factory to create an instance of the given template type.
@@ -544,11 +607,16 @@ public:
      *
      * @return A pointer to the newly created object or a null pointer if the
      *         factory was unable to create an object of the given type.
+     *
+     * @see KPluginFactory::instantiatePlugin
+     * @deprecated Since 5.86, Use KPluginMetaData/KPluginFactory or QPluginloader instead
      */
-    template <class T>
-    T *createInstance(QWidget *parentWidget, QObject *parent,
-                      const QVariantList &args = QVariantList(), QString *error = nullptr) const
+    template<class T>
+    KSERVICE_DEPRECATED_VERSION(5, 86, "Use KPluginMetaData/KPluginFactory or QPluginloader instead")
+    T *createInstance(QWidget *parentWidget, QObject *parent, const QVariantList &args = QVariantList(), QString *error = nullptr) const
     {
+        QT_WARNING_PUSH
+        QT_WARNING_DISABLE_DEPRECATED
         KPluginLoader pluginLoader(*this);
         KPluginFactory *factory = pluginLoader.factory();
         if (factory) {
@@ -557,15 +625,20 @@ public:
             T *o = factory->template create<T>(parentWidget, parent, pluginKeyword(), argsWithMetaData);
             if (!o && error)
                 *error = QCoreApplication::translate("", "The service '%1' does not provide an interface '%2' with keyword '%3'")
-                         .arg(name(), QString::fromLatin1(T::staticMetaObject.className()), pluginKeyword());
+                             .arg(name(), QString::fromLatin1(T::staticMetaObject.className()), pluginKeyword());
             return o;
         } else if (error) {
             *error = pluginLoader.errorString();
             pluginLoader.unload();
         }
+        QT_WARNING_POP
         return nullptr;
     }
+#endif
+#endif
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 86)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 86)
     /**
      * Convert this KService to a KPluginName.
      *
@@ -578,6 +651,25 @@ public:
      * will be set appropriately.
      */
     operator KPluginName() const;
+#endif
+#endif
+
+    /**
+     * @brief A desktop file name that this service is an alias for.
+     *
+     * This is used when a NoDisplay service is used to enforce specific handling
+     * for an application. In that case the NoDisplay service is an AliasFor another
+     * service and be considered roughly equal to the AliasFor service (which should
+     * not be NoDisplay=true)
+     * For example okular supplies a desktop file for each supported format (e.g. PDF), all
+     * of which NoDisplay and merely there to selectively support specific file formats.
+     * A UI may choose to display the aliased entry org.kde.okular instead of the NoDisplay entries.
+     *
+     * @since 5.96
+     *
+     * @return QString desktopName of the aliased service (excluding .desktop suffix)
+     */
+    QString aliasFor() const;
 
 private:
     friend class KBuildServiceFactory;
@@ -585,11 +677,17 @@ private:
     /// @internal for KBuildSycoca only
     struct ServiceTypeAndPreference {
         ServiceTypeAndPreference()
-            : preference(-1), serviceType() {}
+            : preference(-1)
+            , serviceType()
+        {
+        }
         ServiceTypeAndPreference(int pref, const QString &servType)
-            : preference(pref), serviceType(servType) {}
+            : preference(pref)
+            , serviceType(servType)
+        {
+        }
         int preference;
-        QString serviceType; // or mimetype
+        QString serviceType; // or MIME type
     };
     /// @internal for KBuildSycoca only
     QVector<ServiceTypeAndPreference> &_k_accessServiceTypes();
@@ -604,7 +702,7 @@ private:
     /**
      * @internal
      * Construct a service from a stream.
-     * The stream must already be positionned at the correct offset.
+     * The stream must already be positioned at the correct offset.
      */
     KService(QDataStream &str, int offset);
 };
